@@ -16,6 +16,8 @@ from typing import (
     TYPE_CHECKING,
 )
 from pydantic import BaseModel, constr, validate_arguments
+from fastapi import HTTPException
+
 
 if TYPE_CHECKING:
     from roid.app import SlashCommands
@@ -28,11 +30,20 @@ from roid.interactions import (
     CommandOption,
     CommandOptionType,
     CommandChoice,
+    InteractionType,
+)
+from roid.objects import (
+    Role,
+    Channel,
+    Member,
+    PartialMessage,
+    User,
+    ResponseType,
+    CompletedOption,
 )
-from roid.objects import Role, Channel, Member, PartialMessage, User
 from roid.extractors import extract_options
 from roid.checks import CommandCheck
-from roid.response import ResponsePayload
+from roid.response import ResponsePayload, ResponseData
 from roid.callers import OptionalAsyncCallable
 
 
@@ -63,11 +74,13 @@ class SetValue:
         default: Union[str, int, float, None, bool],
         name: Optional[str],
         description: Optional[str],
+        autocomplete: bool,
     ):
         self.default = default
         self.required = self.default is Ellipsis
         self.name = name
         self.description = description
+        self.autocomplete = autocomplete
 
 
 def Option(
@@ -75,8 +88,9 @@ def Option(
     *,
     name: str = None,
     description: str = None,
+    autocomplete: bool = False,
 ) -> Any:  # noqa
-    return SetValue(default, name, description)
+    return SetValue(default, name, description, autocomplete)
 
 
 VALID_CHOICE_TYPES = (
@@ -111,6 +125,51 @@ class PassTarget(IntEnum):
     MEMBER = auto()
 
 
+class AutoCompleteHandler(OptionalAsyncCallable):
+    """
+    A autocomplete option that gets invoked when ever a user starts typing
+    the auto complete field.
+    """
+
+    DEFAULT_TARGET = "_AUTO_COMPLETE_DEFAULT"
+    Callback = Callable[
+        ..., Union[List[CompletedOption], Coroutine[Any, Any, CompletedOption]]
+    ]
+
+    def __init__(
+        self,
+        callback: Callback,
+        target: Optional[str] = None,
+    ):
+        super().__init__(callback)
+
+        self.target = target
+
+        self.target_options = {name for name in self.annotations if name != "return"}
+
+    async def _get_kwargs(
+        self,
+        app: SlashCommands,
+        interaction: Interaction,
+    ) -> dict:
+        kwargs = await super()._get_kwargs(app, interaction)
+
+        if interaction.data.options is None:
+            raise HTTPException(status_code=400)
+
+        for option in interaction.data.options:
+            if self.target is not None and self.target == option.name:
+                kwargs[option.name] = option
+                break
+
+            if self.target is None and len(self.annotations) <= 1:
+                kwargs[option.name] = option
+            elif self.target is None and option.name in self.annotations:
+                kwargs[option.name] = option
+
+        return kwargs
+
+
 class Command(OptionalAsyncCallable):
     def __init__(
         self,
@@ -186,6 +245,7 @@ class Command(OptionalAsyncCallable):
         self.default_permission = default_permissions
         self.options = options if len(options) != 0 else None
 
+        self._autocomplete_handlers: Dict[str, AutoCompleteHandler] = {}
         self._checks_pipeline: List[CommandCheck] = []
 
     def _get_details_from_spec(
@@ -303,6 +363,22 @@ class Command(OptionalAsyncCallable):
         This is naive of any guild ids registered for this command.
         """
 
+        options = self.options
+
+        for option in options:
+            if option.name in self._autocomplete_handlers:
+                option.autocomplete = True
+                continue
+
+            general_handler = self._autocomplete_handlers.get(
+                AutoCompleteHandler.DEFAULT_TARGET
+            )
+            if (
+                general_handler is not None
+                and option.name in general_handler.target_options
+            ):
+                option.autocomplete = True
+
         return CommandContext(
             application_id=self.application_id,
             type=self.type,
@@ -383,8 +459,13 @@ class Command(OptionalAsyncCallable):
         return {**kwargs, **extend}
 
     async def __call__(
-        self, app: SlashCommands, interaction: Interaction
+        self,
+        app: SlashCommands,
+        interaction: Interaction,
     ) -> ResponsePayload:
+        if interaction.type == InteractionType.APPLICATION_COMMAND_AUTOCOMPLETE:
+            return await self._handle_autocomplete(app, interaction)
+
         try:
             for check in self._checks_pipeline:
                 interaction = await check(app, interaction)
@@ -393,6 +474,46 @@ class Command(OptionalAsyncCallable):
 
         return await self._invoke(app, interaction)
 
+    async def _handle_autocomplete(
+        self,
+        app: SlashCommands,
+        interaction: Interaction,
+    ) -> ResponsePayload:
+
+        primary_caller = self._autocomplete_handlers.get(
+            AutoCompleteHandler.DEFAULT_TARGET
+        )
+        if primary_caller is not None:
+            results = await primary_caller(app, interaction)
+            return ResponsePayload(
+                type=ResponseType.APPLICATION_COMMAND_AUTOCOMPLETE_RESULT,
+                data=ResponseData(choices=results),
+            )
+
+        if interaction.data.options is None:
+            raise HTTPException(status_code=400)
+
+        target: Optional[str] = None
+        for option in interaction.data.options:
+            if option.focused:
+                target = option.name
+                break
+
+        if target is None:
+            raise HTTPException(status_code=400)
+
+        secondary_caller = self._autocomplete_handlers.get(target)
+        if secondary_caller is None:
+            raise ValueError(
+                f"no autocomplete handler exists option {secondary_caller!r}."
+            )
+
+        results = await secondary_caller(app, interaction)
+        return ResponsePayload(
+            type=ResponseType.APPLICATION_COMMAND_AUTOCOMPLETE_RESULT,
+            data=ResponseData(choices=results),
+        )
+
     def error(
         self,
         func: Callable[[Interaction, Exception], Coroutine[Any, Any, ResponsePayload]],
@@ -412,6 +533,60 @@ class Command(OptionalAsyncCallable):
         self._register_error_handler(func)
         return func
 
+    def autocomplete(
+        self,
+        func: Optional[AutoCompleteHandler.Callback] = None,
+        *,
+        for_=AutoCompleteHandler.DEFAULT_TARGET,
+    ):
+        """
+        Add a callback for auto complete interaction
+        requests for all or a specific option.
+
+        This decorator can be used either as a generic @command.autocomplete
+        or pass a option target via @command.autocomplete(for_="my_option_name").
+
+        Args:
+            func:
+                The callback for the autocomplete interaction.
+                This is only required when adding a general handler for all options.
+            for_:
+                A optional name to target a specific option.
+                If this is given the callback will only be invoked if the value is
+                focused.
+                The callback required will also just be given the raw `value: str`
+                keyword opposed to a set of kwargs.
+        """
+        if func is not None:
+            existing = self._autocomplete_handlers.get(for_)
+            if existing:
+                raise ValueError(
+                    f"autocomplete handler {for_!r} registered already as callable: {existing._callback!r}."
+                )
+
+            target = for_ if for_ != AutoCompleteHandler.DEFAULT_TARGET else None
+            self._autocomplete_handlers[for_] = AutoCompleteHandler(func, target=target)
+            return func
+
+        if for_ not in self.original_annotations:
+            if for_ != AutoCompleteHandler.DEFAULT_TARGET:
+                raise ValueError(
+                    f"there is no parameter named {for_!r} in the command's signature."
+                )
+            raise TypeError(f"missing required keyword parameter 'for_'.")
+
+        existing = self._autocomplete_handlers.get(AutoCompleteHandler.DEFAULT_TARGET)
+        if existing:
+            raise ValueError(
+                f"general autocomplete handler registered already as callable: {existing._callback!r}."
+            )
+
+        def wrapper(func_):
+            self._autocomplete_handlers[for_] = AutoCompleteHandler(func_, target=for_)
+            return func_
+
+        return wrapper
+
 
 class GroupCommand(Command):
     def __init__(self, app: SlashCommands, callback, name: str, application_id: int):
